@@ -162,10 +162,11 @@ DataImpl :: load_newsrc_files (const DataIO& data_io)
   s.clear ();
   u.clear ();
 
-  const quarks_t servers (data_io.get_newsrc_servers ());
-  foreach_const (quarks_t, servers, sit) {
-    LineReader * in (data_io.read_newsrc (*sit));
-    load_newsrc (*sit, in, s, u);
+  foreach_const (servers_t, _servers, sit) {
+    const Quark key (sit->first);
+    const std::string filename (sit->second.newsrc_filename);
+    LineReader * in (data_io.read_file (filename));
+    load_newsrc (key, in, s, u);
     delete in;
   }
 
@@ -202,7 +203,8 @@ DataImpl :: save_newsrc_files (DataIO& data_io) const
     const Quark& server (sit->first);
 
     // write this server's newsrc
-    std::ostream * out (data_io.write_newsrc (server));
+    const std::string& filename (sit->second.newsrc_filename);
+    std::ostream * out (data_io.write_file (filename));
     std::string newsrc_string;
     alpha_groups_t::const_iterator sub_it (_subscribed.begin());
     const alpha_groups_t::const_iterator sub_end(_subscribed.end());
@@ -294,7 +296,17 @@ DataImpl :: save_group_permissions (DataIO& data_io) const
 ***/
 
 void
-DataImpl :: load_group_descriptions (const DataIO& data_io)
+DataImpl :: ensure_descriptions_are_loaded () const
+{
+  if (!_descriptions_loaded)
+  {
+    _descriptions_loaded = true;
+    load_group_descriptions (*_data_io);
+  }
+}
+
+void
+DataImpl :: load_group_descriptions (const DataIO& data_io) const
 {
   _descriptions.clear ();
 
@@ -342,6 +354,8 @@ DataImpl :: load_group_xovers (const DataIO& data_io)
 void
 DataImpl :: save_group_descriptions (DataIO& data_io) const
 {
+  assert (_descriptions_loaded);
+
   typedef std::map<Quark, std::string, AlphabeticalQuarkOrdering> tmp_t;
   tmp_t tmp;
   foreach_const (descriptions_t, _descriptions, it)
@@ -434,6 +448,8 @@ DataImpl :: add_groups (const Quark       & server,
                         const NewGroup    * newgroups,
                         size_t              count)
 {
+  ensure_descriptions_are_loaded ();
+
   Server * s (find_server (server));
   assert (s != 0);
 
@@ -545,6 +561,7 @@ DataImpl :: set_group_subscribed (const Quark& group, bool subscribed)
 const std::string&
 DataImpl :: get_group_description (const Quark& group) const
 {
+  ensure_descriptions_are_loaded ();
   static const std::string nil;
   descriptions_t::const_iterator it (_descriptions.find (group));
   return it == _descriptions.end() ? nil : it->second;
