@@ -217,7 +217,7 @@ ProfilesImpl :: serialize (std::ostream& out) const
     out << indent(depth++) << "<profile name=\"" << escaped(it->first) << "\">\n";
     out << indent(depth) << "<username>" << escaped(it->second.username) << "</username>\n";
     out << indent(depth) << "<address>" << escaped(it->second.address) << "</address>\n";
-    out << indent(depth) << "<server>" << escaped(it->second.posting_server.to_string()) << "</server>\n";
+    out << indent(depth) << "<server>" << escaped(it->second.posting_server.to_view()) << "</server>\n";
     if (!it->second.signature_file.empty()) {
       const char * type;
       switch (it->second.sig_type) {
@@ -313,12 +313,10 @@ ProfilesImpl :: has_profiles () const
 bool
 ProfilesImpl :: has_from_header (const StringView& from) const
 {
-  std::string s;
-  foreach_const (profiles_t, profiles, it) {
-    it->second.get_from_header (s);
-    if (from == s)
+  foreach_const (profiles_t, profiles, it)
+    if (from.strstr (it->second.address))
       return true;
-  }
+
   return false;
 }
 
