@@ -137,7 +137,7 @@ namespace
 *****/
 
 ArticleCache :: ArticleCache (const StringView& path, size_t max_megs):
-   _path (path.to_string()),
+   _path (path.str, path.len),
    _max_megs (max_megs),
    _current_bytes (0ul)
 {
@@ -213,7 +213,7 @@ ArticleCache :: get_filename (char * buf, int buflen, const Quark& mid) const
 {
    char basename[PATH_MAX];
    *buf = '\0';
-   message_id_to_filename (basename, sizeof(basename), mid.to_string());
+   message_id_to_filename (basename, sizeof(basename), mid.to_view());
    g_snprintf (buf, buflen, "%s%c%s", _path.c_str(), G_DIR_SEPARATOR, basename);
    return buf && *buf ? buf : 0;
 };
@@ -266,34 +266,16 @@ ArticleCache :: add (const Quark& message_id, const StringView& article)
 ***/
 
 void
-ArticleCache :: reserve (const Article::parts_t& parts)
+ArticleCache :: reserve (const mid_sequence_t& mids)
 {
-  quarks_t mids;
-  foreach_const (Article::parts_t, parts, it)
-    mids.insert (it->message_id);
-  reserve (mids);
-}
-
-void
-ArticleCache :: reserve (const quarks_t& mids)
-{
-  for (quarks_t::const_iterator it=mids.begin(), end=mids.end(); it!=end; ++it)
+  foreach_const (mid_sequence_t, mids, it)
     ++_locks[*it];
 }
 
 void
-ArticleCache :: release (const Article::parts_t& parts)
+ArticleCache :: release (const mid_sequence_t& mids)
 {
-  quarks_t mids;
-  foreach_const (Article::parts_t, parts, it)
-    mids.insert (it->message_id);
-  release (mids);
-}
-
-void
-ArticleCache :: release (const quarks_t& mids)
-{
-  for (quarks_t::const_iterator it=mids.begin(), end=mids.end(); it!=end; ++it)
+  foreach_const (mid_sequence_t, mids, it)
     if (!--_locks[*it])
       _locks.erase (*it);
 }
@@ -302,7 +284,6 @@ ArticleCache :: release (const quarks_t& mids)
 ****
 ***/
 
-
 void
 ArticleCache :: expire_to_size ()
 {
@@ -310,7 +291,18 @@ ArticleCache :: expire_to_size ()
   const double buffer_zone (0.8);
   size_t max_bytes (_max_megs * 1024 * 1024);
   max_bytes = (size_t) ((double)max_bytes * buffer_zone);
+  expire_to_size (max_bytes);
+}
 
+void
+ArticleCache :: clear ()
+{
+  expire_to_size (0);
+}
+
+void
+ArticleCache :: expire_to_size (size_t max_bytes)
+{
   quarks_t removed;
   if (_current_bytes > max_bytes)
   {
@@ -434,23 +426,13 @@ ArticleCache :: get_message (const mid_sequence_t& mids)
    return retval;
 }
 
-GMimeMessage*
-ArticleCache :: get_message (const Article::parts_t& parts)
-{
-   mid_sequence_t mids;
-   for (Article::parts_t::const_iterator it=parts.begin(), end=parts.end(); it!=end; ++it)
-      mids.push_back (it->message_id);
-   return get_message (mids);
-}
-
 ArticleCache :: strings_t
-ArticleCache :: get_filenames (const Article::parts_t& parts)
+ArticleCache :: get_filenames (const mid_sequence_t& mids)
 {
   strings_t ret;
   char filename[PATH_MAX];
-  for (Article::parts_t::const_iterator it=parts.begin(), end=parts.end(); it!=end; ++it)
-    if (get_filename (filename, sizeof(filename), it->message_id))
+  foreach_const (mid_sequence_t, mids, it)
+    if (get_filename (filename, sizeof(filename), *it))
       ret.push_back (filename);
   return ret;
 }
-
