@@ -24,7 +24,6 @@
 #include <vector>
 #include <pan/general/string-view.h>
 #include <pan/general/quark.h>
-#include "article.h"
 
 extern "C"
 {
@@ -59,20 +58,19 @@ namespace pan
       ArticleCache (const StringView& path, size_t max_megs=10);
       ~ArticleCache ();
 
+      typedef std::vector<Quark> mid_sequence_t;
+
       bool contains (const Quark& message_id) const;
       void add (const Quark& message_id, const StringView& article);
-      void reserve (const quarks_t& mids);
-      void release (const quarks_t& mids);
-      void reserve (const Article::parts_t&);
-      void release (const Article::parts_t&);
+      void reserve (const mid_sequence_t& mids);
+      void release (const mid_sequence_t& mids);
       void expire_to_size ();
+      void clear ();
 
-      typedef std::vector<Quark> mid_sequence_t;
       GMimeMessage* get_message (const mid_sequence_t&);
-      GMimeMessage* get_message (const Article::parts_t&);
 
       typedef std::vector<std::string> strings_t;
-      strings_t get_filenames (const Article::parts_t&);
+      strings_t get_filenames (const mid_sequence_t&);
 
     public:
 
@@ -101,7 +99,9 @@ namespace pan
 
       struct MsgInfoCompare {
         bool operator()(const MsgInfo& a, const MsgInfo& b) const {
-          return a._date < b._date;
+          if (a._date != b._date)
+            return a._date < b._date;
+          return a._message_id < b._message_id;
         }
       };
 
@@ -115,6 +115,8 @@ namespace pan
       void fire_added (const Quark& mid);
       void fire_removed (const quarks_t& mid);
 
+      void expire_to_size (size_t max_bytes);
+
       char* get_filename (char* buf, int buflen, const Quark& mid) const;
       GMimeStream* get_message_file_stream (const Quark& mid) const;
       GMimeStream* get_message_mem_stream (const Quark& mid) const;
