@@ -48,7 +48,6 @@ namespace
 
   const BuiltinIconInfo my_builtin_icons [] =
   {
-    { icon_article_new, "ICON_ARTICLE_NEW" },
     { icon_article_read, "ICON_ARTICLE_READ" },
     { icon_article_unread, "ICON_ARTICLE_UNREAD" },
     { icon_bluecheck, "ICON_BLUECHECK" }, 
@@ -59,19 +58,15 @@ namespace
     { icon_filter_only_attachments, "ICON_ONLY_ATTACHMENTS" }, 
     { icon_filter_only_cached, "ICON_ONLY_CACHED" }, 
     { icon_filter_only_me, "ICON_ONLY_ME" }, 
-    { icon_filter_only_new, "ICON_ONLY_NEW" }, 
+    { icon_filter_only_unread, "ICON_ONLY_UNREAD" }, 
     { icon_filter_only_watched, "ICON_ONLY_WATCHED" }, 
     { icon_get_dialog, "ICON_GET_DIALOG" }, 
-    { icon_get_new, "ICON_GET_NEW" }, 
     { icon_get_selected, "ICON_GET_SELECTED" }, 
     { icon_get_subscribed, "ICON_GET_SUBSCRIBED" }, 
-    { icon_online, "ICON_ONLINE" }, 
     { icon_pan, "ICON_PAN" }, 
     { icon_read_group, "ICON_READ_GROUP" }, 
     { icon_read_more, "ICON_READ_MORE" }, 
     { icon_read_less, "ICON_READ_LESS" }, 
-    { icon_read_new_article, "ICON_READ_NEW_ARTICLE" }, 
-    { icon_read_new_thread, "ICON_READ_NEW_THREAD" }, 
     { icon_read_unread_article, "ICON_READ_UNREAD_ARTICLE" }, 
     { icon_read_unread_thread, "ICON_READ_UNREAD_THREAD" }, 
     { icon_score, "ICON_SCORE" }, 
@@ -139,6 +134,7 @@ namespace
   void do_rot13_selected_text          (GtkAction * a) { pan_ui->do_rot13_selected_text(); }
   void do_download_selected_article    (GtkAction * a) { pan_ui->do_download_selected_article(); }
   void do_read_selected_article        (GtkAction * a) { pan_ui->do_read_selected_article(); }
+  void do_show_selected_article_info   (GtkAction * a) { pan_ui->do_show_selected_article_info(); }
   void do_read_more                    (GtkAction * a) { pan_ui->do_read_more(); }
   void do_read_less                    (GtkAction * a) { pan_ui->do_read_less(); }
   void do_read_next_unread_group       (GtkAction * a) { pan_ui->do_read_next_unread_group(); }
@@ -160,6 +156,7 @@ namespace
   void do_cancel_article               (GtkAction * a) { pan_ui->do_cancel_article(); }
   void do_supersede_article            (GtkAction * a) { pan_ui->do_supersede_article(); }
   void do_delete_article               (GtkAction * a) { pan_ui->do_delete_article(); }
+  void do_clear_article_cache          (GtkAction * a) { pan_ui->do_clear_article_cache(); }
   void do_mark_article_read            (GtkAction * a) { pan_ui->do_mark_article_read(); }
   void do_mark_article_unread          (GtkAction * a) { pan_ui->do_mark_article_unread(); }
   void do_post                         (GtkAction * a) { pan_ui->do_post(); }
@@ -311,6 +308,7 @@ namespace
 
     { "download-selected-article", "ICON_DISK", N_("Cache Article"), NULL, N_("Cache Article"), G_CALLBACK(do_download_selected_article) },
     { "read-selected-article", "ICON_READ_MORE", N_("Read Article"), NULL, N_("Read Article"), G_CALLBACK(do_read_selected_article) },
+    { "show-selected-article-info", NULL, N_("Show Article Information"), NULL, N_("Show Article Information"), G_CALLBACK(do_show_selected_article_info) },
     { "read-more", "ICON_READ_MORE", N_("Read _More"), "space", N_("Read More"), G_CALLBACK(do_read_more) },
     { "read-less", "ICON_READ_LESS", N_("Read _Back"), "BackSpace", N_("Read Back"), G_CALLBACK(do_read_less) },
     { "read-next-unread-group", "ICON_READ_UNREAD_GROUP", N_("Next _Unread Group"), "G", N_("Next Unread Group"), G_CALLBACK(do_read_next_unread_group) },
@@ -329,9 +327,10 @@ namespace
     { "ignore-thread", NULL, N_("_Ignore Thread"), NULL, NULL, G_CALLBACK(do_ignore) },
     { "view-article-score", "ICON_SCORE", N_("Edit Article's Watch/Ignore/Score ..."), "<control><shift>C", NULL, G_CALLBACK(do_show_score_dialog) },
     { "add-article-score", "ICON_SCORE", N_("_Add a _Scoring Rule ..."), "S", NULL, G_CALLBACK(do_show_new_score_dialog) },
-    { "cancel-article", GTK_STOCK_CANCEL, N_("Cance_l Article ..."), NULL, NULL, G_CALLBACK(do_cancel_article) },
-    { "supersede-article", GTK_STOCK_CONVERT, N_("_Supersede Article ..."), NULL, NULL, G_CALLBACK(do_supersede_article) },
+    { "cancel-article", NULL, N_("Cance_l Article ..."), NULL, NULL, G_CALLBACK(do_cancel_article) },
+    { "supersede-article", NULL, N_("_Supersede Article ..."), NULL, NULL, G_CALLBACK(do_supersede_article) },
     { "delete-article", GTK_STOCK_DELETE, N_("_Delete Article"), "Delete", NULL, G_CALLBACK(do_delete_article) },
+    { "clear-article-cache", NULL, N_("Clear Article Cache"), NULL, NULL, G_CALLBACK(do_clear_article_cache) },
     { "mark-article-read", "ICON_ARTICLE_READ", N_("_Mark Article as Read"), "M", N_("Mark Article as Read"), G_CALLBACK(do_mark_article_read) },
     { "mark-article-unread", "ICON_ARTICLE_UNREAD", N_("Mark Article as _Unread"), "<control>M", N_("Mark Article as Unread"), G_CALLBACK(do_mark_article_unread) },
 
@@ -367,6 +366,8 @@ namespace
     { "show-text-markup",         NULL, N_("Show *Bold*, __Underlined__, and /Italic/"), NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "size-pictures-to-fit",     NULL, N_("Size Pictures to _Fit"),          NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "monospace-font-enabled",   NULL, N_("Use _Monospace Font"),             "C", NULL, G_CALLBACK(prefs_toggle_callback_impl), false },
+    { "focus-on-image",           NULL, N_("Set Focus to Images"),            NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
+
 
     { "work-online", "ICON_ONLINE", N_("_Work Online"), "L", NULL, G_CALLBACK(do_work_online), true },
     { "tabbed-layout", GTK_STOCK_JUMP_TO, N_("_Tabbed Layout"), "Z", NULL, G_CALLBACK(do_tabbed_layout), false },
@@ -376,7 +377,7 @@ namespace
     { "show-toolbar", NULL, N_("Show _Toolbar"), NULL, NULL, G_CALLBACK(do_show_toolbar), true },
     { "shorten-group-names", GTK_STOCK_ZOOM_OUT, N_("Abbreviate _Names in Group Pane"), "B", N_("Abbreviate Newsgroup Names"), G_CALLBACK(do_shorten_group_names), false },
 
-    { "match-only-unread-articles", "ICON_ONLY_NEW", N_("Match Only _Unread Articles"), NULL, N_("Match Only Unread Articles"), G_CALLBACK(do_match_only_unread_articles), false },
+    { "match-only-unread-articles", "ICON_ONLY_UNREAD", N_("Match Only _Unread Articles"), NULL, N_("Match Only Unread Articles"), G_CALLBACK(do_match_only_unread_articles), false },
     { "match-only-cached-articles", "ICON_ONLY_CACHED", N_("Match Only _Cached Articles"), NULL, N_("Match Only Cached Articles"), G_CALLBACK(do_match_only_cached_articles), false },
     { "match-only-binary-articles", "ICON_ONLY_ATTACHMENTS", N_("Match Only _Complete Articles"), NULL, N_("Match Only Complete Articles"), G_CALLBACK(do_match_only_binary_articles), false },
     { "match-only-my-articles", "ICON_ONLY_ME", N_("Match Only _My Articles"), NULL, N_("Match Only Articles Posted by Me"), G_CALLBACK(do_match_only_my_articles), false },
@@ -500,6 +501,7 @@ pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p)
                                       18, // UTF-8
                                       G_CALLBACK(charset_cb), NULL);
   gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+  g_object_unref (G_OBJECT(action_group));
 
 #if 1
   int merge_id = gtk_ui_manager_new_merge_id (ui_manager);
