@@ -40,7 +40,6 @@ namespace
 {
   enum ModelColumns
   {
-    COL_TOTAL,
     COL_UNREAD,
     COL_QTY
   };
@@ -49,13 +48,11 @@ namespace
   {
     public:
       Quark groupname;
-      unsigned long total;
       unsigned long unread;
 
     public:
       virtual void get_value (int column, GValue* setme) {
         switch (column) {
-          case COL_TOTAL:  set_value_ulong   (setme, total);  break;
           case COL_UNREAD: set_value_ulong   (setme, unread); break;
         }
       }
@@ -95,20 +92,6 @@ namespace
     return out;
   }
 
-  /**
-   * WARNING: the model holds a pointer to the groupname Quark, so it must be
-   * persistent for the life of this model -- a temporary will not do!!
-   * For that reason, callers should pass in a reference to a groups_in_model element:
-   *   create_row (_tree_store, &iter, _data, *groups_in_model.insert(group).first);
-   */
-  MyRow* create_row (const Data& data, const Quark& groupname)
-  {
-    MyRow * row = new MyRow ();
-    row->groupname = groupname;
-    data.get_group_counts (groupname, row->unread, row->total);
-    return row;
-  }
-
   void
   find_matching_groups (const TextMatch           * match,
                         const std::vector<Quark>  & sorted_groups,
@@ -298,8 +281,40 @@ GroupPane :: on_popup_menu (GtkWidget *treeview, gpointer userdata)
 namespace
 {
   typedef std::vector<GtkTreeIter> tree_iters_t;
-  typedef std::pair<Quark,MyRow*> row_t;
-  typedef Loki::AssocVector<Quark,MyRow*> rows_t;
+
+  struct MyRowLessThan {
+    bool operator() (const MyRow* a, const MyRow* b) const {
+      return a->groupname < b->groupname;
+    }
+  };
+
+  typedef sorted_vector<MyRow*, true, MyRowLessThan> rows_t;
+
+  rows_t _group_rows;
+
+  MyRow* find_row (const Quark& groupname)
+  {
+    MyRow tmp;
+    tmp.groupname = groupname;
+    rows_t::iterator it = _group_rows.find (&tmp);
+    return it==_group_rows.end() ? 0 : *it;
+  }
+
+  void
+  delete_rows (gpointer data, GObject * dead_object)
+  {
+    delete [] static_cast<MyRow*>(data);
+  }
+
+  // give PanTreeStore a noop row dispose functor
+  // since we allocate our rows in a block.
+  struct NoopRowDispose: public PanTreeStore::RowDispose
+  {
+    virtual ~NoopRowDispose () { }
+    virtual void row_dispose (PanTreeStore *store, PanTreeStore::Row* row) { }
+  };
+
+  NoopRowDispose noop_row_dispose;
 
   PanTreeStore*
   build_model (const Data         & data,
@@ -308,9 +323,8 @@ namespace
                rows_t             & setme_rows)
   {
     // build the model...
-    PanTreeStore * store = PanTreeStore :: new_tree (COL_QTY, 
-                                                     G_TYPE_ULONG,   // total
-                                                     G_TYPE_ULONG);  // unread
+    PanTreeStore * store = PanTreeStore :: new_tree (COL_QTY, G_TYPE_ULONG);  // unread
+    store->set_row_dispose (&noop_row_dispose);
 
     // find the groups that we'll be adding.
     std::vector<Quark> groups, sub, unsub;
@@ -320,38 +334,65 @@ namespace
     data.get_subscribed_groups (groups);
     find_matching_groups (match, groups, sub);
 
-    std::vector<row_t> tmp_rows;
-    tmp_rows.reserve (sub.size() + unsub.size());
+    std::vector<MyRow*>& group_rows (setme_rows.get_container());
+    group_rows.clear ();
+    group_rows.reserve (sub.size() + unsub.size());
 
-    // subscribed
-    MyRow * row = new MyRow ();
-    row->groupname = *sub_title_quark;
+    MyRow * headers = new MyRow[2];
+    headers[0].groupname = *sub_title_quark;
+    headers[1].groupname = *other_title_quark;
+    g_object_weak_ref (G_OBJECT(store), delete_rows, headers);
+ 
+    // 
+    //  subscribed
+    //
+
+    MyRow * row = &headers[0];
     store->append (NULL, row);
-    if (!sub.empty()) {
-      std::vector<PanTreeStore::Row*> tmp (sub.size());
-      for (size_t i(0), n(sub.size()); i!=n; ++i) {
-        const Quark& groupname (sub[i]);
-        MyRow * r = create_row (data, groupname);
-        tmp[i] = r;
-        tmp_rows.push_back (row_t (groupname, r));
+    if (!sub.empty())
+    {
+      const size_t n (sub.size());
+      std::vector<PanTreeStore::Row*> appendme;
+      appendme.reserve (n);
+      MyRow *rows(new MyRow [n]), *r(rows);
+      g_object_weak_ref (G_OBJECT(store), delete_rows, rows);
+
+      unsigned long unused;
+      for (size_t i(0); i!=n; ++i, ++r) {
+        r->groupname = sub[i];
+        data.get_group_counts (r->groupname, r->unread, unused);
+        appendme.push_back (r);
+        group_rows.push_back (r);
       }
-      store->append (row, tmp);
+
+      store->append (row, appendme);
       expandme.push_back (store->get_iter (row));
     }
 
+    //
     // unsubscribed
-    row = new MyRow ();
-    row->groupname = *other_title_quark;
+    //
+
+    row = &headers[1];
     store->append (NULL, row);
-    if (!unsub.empty()) {
-      std::vector<PanTreeStore::Row*> tmp (unsub.size());
-      for (size_t i(0), n(unsub.size()); i!=n; ++i) {
-        const Quark& groupname (unsub[i]);
-        MyRow * r = create_row (data, groupname);
-        tmp[i] = r;
-        tmp_rows.push_back (row_t (groupname, r));
+    if (!unsub.empty())
+    {
+      const size_t n (unsub.size());
+      std::vector<PanTreeStore::Row*> appendme;
+      appendme.reserve (n);
+      MyRow *rows(new MyRow[n]), *r(rows);
+      g_object_weak_ref (G_OBJECT(store), delete_rows, rows);
+
+      unsigned long unused;
+      for (size_t i(0); i!=n; ++i, ++r) {
+        r->groupname = unsub[i];
+        data.get_group_counts (r->groupname, r->unread, unused);
+        appendme.push_back (r);
+        group_rows.push_back (r);
       }
-      store->append (row, tmp);
+
+      store->append (row, appendme);
+
       // only expand the unsub expander if user specified search criteria,
       // or if there's no criteria and no subscribed groups (first-time usrs).
       // otherwise it's a flood of thousands of groups.
@@ -361,7 +402,7 @@ namespace
         expandme.push_back (store->get_iter (row));
     }
 
-    setme_rows.assign (tmp_rows.begin(), tmp_rows.end());
+    setme_rows.sort ();
     return store;
   }
 }
@@ -388,7 +429,7 @@ GroupPane :: on_group_subscribe (const Quark& groupname, bool sub)
   }
   
   // move the row
-  _tree_store->reparent (_tree_store->get_row(&section_iter), _group_rows[groupname], pos);
+  _tree_store->reparent (_tree_store->get_row(&section_iter), find_row(groupname), pos);
 
   // make sure it's visible...
   if (sub) {
@@ -403,11 +444,11 @@ GroupPane :: refresh_dirty_groups ()
 {
   foreach_const (quarks_t, _dirty_groups, it)
   {
-    group_rows_t::iterator row_it (_group_rows.find (*it));
-    if (row_it != _group_rows.end())
+    MyRow * row (find_row (*it));
+    if (row)
     {
-      MyRow * row (row_it->second);
-      _data.get_group_counts (row->groupname, row->unread, row->total);
+      unsigned long unused;
+      _data.get_group_counts (row->groupname, row->unread, unused);
       _tree_store->row_changed (row);
     }
   }
@@ -742,8 +783,10 @@ GroupPane :: create_filter_entry ()
   refresh_search_entry (entry);
   gtk_box_pack_start (GTK_BOX(h), entry, 0, 0, false);
 
-  GtkTooltips * tips = gtk_tooltips_new ();
   GtkWidget * w = gtk_button_new ();
+  GtkTooltips * tips = gtk_tooltips_new ();
+  g_object_ref_sink_pan (G_OBJECT(tips));
+  g_object_weak_ref (G_OBJECT(w), (GWeakNotify)g_object_unref, tips);
   g_object_set_data (G_OBJECT(w), "entry", entry);
   g_object_set_data (G_OBJECT(entry), "reset-button", w);
   gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
@@ -754,11 +797,6 @@ GroupPane :: create_filter_entry ()
   gtk_box_pack_start (GTK_BOX(h), w, 0, 0, false);
   gtk_widget_set_sensitive (w, false);
 
-  // when the button's destroyed, free the tips too
-  g_object_ref (tips);
-  gtk_object_sink (GTK_OBJECT (tips));
-  g_signal_connect_swapped (w, "destroy", G_CALLBACK(g_object_unref), tips);
-
   return h;
 }
 
