@@ -20,10 +20,12 @@
 #include <config.h>
 #include <map>
 #include <string>
+#include <sstream>
 extern "C" {
   #include <glib/gi18n.h>
 }
 #include <pan/general/debug.h>
+#include <pan/general/e-util.h>
 #include <pan/general/file-util.h>
 #include <pan/general/foreach.h>
 #include <pan/data/scorefile.h>
@@ -55,6 +57,20 @@ extern "C" {
 #include "task-pane.h"
 #include "url.h"
 
+namespace pan
+{
+  void
+  g_object_ref_sink_pan (GObject * o)
+  {
+#if GLIB_CHECK_VERSION(2,10,0)
+    g_object_ref_sink (o);
+#else
+    g_object_ref (o);
+    gtk_object_sink (GTK_OBJECT(o));
+#endif
+  }
+}
+
 using namespace pan;
 
 namespace
@@ -155,6 +171,9 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
   _taskbar (0),
   _ttips (gtk_tooltips_new ())
 {
+  g_object_ref_sink_pan (G_OBJECT(_ttips));
+  g_object_weak_ref (G_OBJECT(_root), (GWeakNotify)g_object_unref, _ttips);
+
   char * filename = g_build_filename (file::get_pan_home().c_str(), "pan.ui", NULL);
 #if 0
   GError * gerr (0);
@@ -297,10 +316,6 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
 
   _prefs.add_listener (this);
 
-  g_object_ref (_ttips);
-  gtk_object_sink (GTK_OBJECT(_ttips));
-  g_signal_connect_swapped (_root, "destroy", G_CALLBACK(g_object_unref), _ttips);
-
   gtk_accel_map_load (get_accel_filename().c_str());
 }
 
@@ -343,6 +358,7 @@ GUI :: ~GUI ()
 
   foreach (std::set<GtkWidget*>, unref, it)
     gtk_widget_unref (*it);
+  g_object_unref (G_OBJECT(_ui_manager));
 }
 
 /***
@@ -475,30 +491,33 @@ GUI :: disable_accelerators_when_focused (GtkWidget * w) const
 namespace
 {
   static std::string prev_path;
+}
 
-  std::string get_save_attachments_path_from_user (GtkWindow * parent, const Prefs& prefs)
-  {
-    if (prev_path.empty())
-      prev_path = prefs.get_string ("default-save-attachments-path", g_get_home_dir ());
+std::string
+GUI :: prompt_user_for_save_path (GtkWindow * parent, const Prefs& prefs)
+{
+  if (prev_path.empty())
+    prev_path = prefs.get_string ("default-save-attachments-path", g_get_home_dir ());
+  if (!file :: file_exists (prev_path.c_str()))
+    prev_path = g_get_home_dir ();
 
-    GtkWidget * w = gtk_file_chooser_dialog_new (_("Save Attachments"), parent,
-                                                 GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                                 GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-                                                 NULL);
-    gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_ACCEPT);
-    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(w), prev_path.c_str());
-    const int response (gtk_dialog_run (GTK_DIALOG(w)));
-    std::string path;
-    if (response == GTK_RESPONSE_ACCEPT) {
-      char * tmp = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
-      prev_path = path = tmp;
-      g_free (tmp);
-    }
-
-    gtk_widget_destroy(w);
-    return path;
+  GtkWidget * w = gtk_file_chooser_dialog_new (_("Save NZB's Files"), parent,
+                                               GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                                               GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                               GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                                               NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_ACCEPT);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(w), prev_path.c_str());
+  const int response (gtk_dialog_run (GTK_DIALOG(w)));
+  std::string path;
+  if (response == GTK_RESPONSE_ACCEPT) {
+    char * tmp = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
+    prev_path = path = tmp;
+    g_free (tmp);
   }
+
+  gtk_widget_destroy(w);
+  return path;
 }
 
 void GUI :: do_save_articles ()
@@ -548,9 +567,9 @@ void GUI :: do_import_tasks ()
   gtk_widget_destroy (dialog);
 
   // if we're importing files, build the tasks...
-  std::vector<Task*> tasks;
+  Queue::tasks_t tasks;
   if (!filenames.empty()) {
-    const std::string path (get_save_attachments_path_from_user (get_window(_root), _prefs));
+    const std::string path (prompt_user_for_save_path (get_window(_root), _prefs));
     if (!path.empty())
       foreach_const (strings_t, filenames, it)
         NZB :: tasks_from_nzb_file (*it, path, _cache, _data, _data, _data, tasks);
@@ -727,10 +746,13 @@ void GUI :: do_read_selected_article ()
 }
 void GUI :: do_download_selected_article ()
 {
-  typedef std::set<const Article*> article_set_t;
-  const article_set_t articles (_header_pane->get_full_selection ());
-  foreach_const (article_set_t, articles, it)
-    _queue.add_task (new TaskArticle (_data, _data, **it, _cache, _data, this));
+  typedef std::vector<const Article*> article_vector_t;
+  const article_vector_t articles (_header_pane->get_full_selection_v ());
+  Queue::tasks_t tasks;
+  foreach_const (article_vector_t, articles, it)
+    tasks.push_back (new TaskArticle (_data, _data, **it, _cache, _data, this));
+  if (!tasks.empty())
+    _queue.add_tasks (tasks, Queue::TOP);
 }
 void GUI :: do_clear_header_pane ()
 {
@@ -903,6 +925,7 @@ void GUI :: do_supersede_article ()
   // did this user post the message?
   const char * sender (g_mime_message_get_sender (message));
   const bool user_posted_this (_data.has_from_header (sender));
+  
   if (!user_posted_this) {
     GtkWidget * w = gtk_message_dialog_new (
       get_window(_root),
@@ -928,6 +951,7 @@ void GUI :: do_supersede_article ()
   g_mime_message_set_sender (new_message, g_mime_message_get_sender (message));
   g_mime_message_set_subject (new_message, g_mime_message_get_subject (message));
   g_mime_message_set_header (new_message, "Newsgroups", g_mime_message_get_header (message, "Newsgroups"));
+  g_mime_message_set_header (new_message, "References", g_mime_message_get_header (message, "References"));
   if ((cpch = g_mime_message_get_reply_to (message)))
               g_mime_message_set_reply_to (new_message, cpch);
   if ((cpch = g_mime_message_get_header (message,     "Followup-To")))
@@ -1036,6 +1060,12 @@ void GUI :: do_delete_article ()
     if ((*it)->message_id == mid)
       _body_pane->clear ();
 }
+
+void GUI :: do_clear_article_cache ()
+{
+  _cache.clear ();
+}
+
 void GUI :: do_mark_article_read ()
 {
   const std::set<const Article*> article_set (_header_pane->get_nested_selection ());
@@ -1114,6 +1144,8 @@ void GUI :: do_about_pan ()
   const gchar * authors [] = { "Charles Kerr", 0 };
   GdkPixbuf * logo = gdk_pixbuf_new_from_inline(-1, icon_pan_about_logo, 0, 0);
   GtkAboutDialog * w (GTK_ABOUT_DIALOG (gtk_about_dialog_new ()));
+  GdkPixbuf * icon = gdk_pixbuf_new_from_inline (-1, icon_pan, FALSE, 0);
+  gtk_window_set_icon (GTK_WINDOW(w), icon); 
   gtk_about_dialog_set_name (w, _("Pan"));
   gtk_about_dialog_set_version (w, PACKAGE_VERSION);
   gtk_about_dialog_set_comments (w, VERSION_TITLE);
@@ -1126,6 +1158,7 @@ void GUI :: do_about_pan ()
   g_signal_connect (G_OBJECT (w), "response", G_CALLBACK (gtk_widget_destroy), NULL);
   gtk_widget_show_all (GTK_WIDGET(w));
   g_object_unref (logo);
+  g_object_unref (icon);
 #else
   GtkWidget * dialog = gtk_dialog_new_with_buttons (PACKAGE_STRING,
                                                     GTK_WINDOW (get_window (_root)),
@@ -1308,6 +1341,59 @@ void GUI :: do_show_matches (const Data::ShowType show_type)
 {
   _header_pane->set_show_type (show_type);
 }
+
+void GUI :: do_show_selected_article_info ()
+{
+  const Article* a = _header_pane->get_first_selected_article ();
+
+  if (a != 0)
+  {
+    // date
+    EvolutionDateMaker date_maker;
+    char * date = date_maker.get_date_string (a->time_posted);
+
+    // article parts
+    std::vector<int> missing_parts;
+    int i = 0;
+    foreach_const (Article::parts_t, a->parts, it) {
+      ++i;
+      if (it->empty())
+        missing_parts.push_back (i);
+    }
+    char msg[512];
+    *msg = '\0';
+    std::ostringstream s;
+    if (i > 1) {
+      if (missing_parts.empty())
+        g_snprintf (msg, sizeof(msg), _("This article has all %d parts."), i);
+      else
+        g_snprintf (msg, sizeof(msg), _("This article is missing %d of its %d parts:"), (int)missing_parts.size(), i);
+      foreach_const (std::vector<int>, missing_parts, it)
+        s << ' ' << *it;
+    }
+
+    GtkWidget * w = gtk_message_dialog_new_with_markup (
+      get_window(_root),
+      GTK_DIALOG_DESTROY_WITH_PARENT,
+      GTK_MESSAGE_INFO,
+      GTK_BUTTONS_CLOSE,
+        "<b>%s</b>: %s\n" "<b>%s</b>: %s\n"
+        "<b>%s</b>: %s\n" "<b>%s</b>: %s\n"
+        "<b>%s</b>: %lu\n" "<b>%s</b>: %lu\n"
+        "\n"
+        "%s" "%s",
+        _("Subject"), a->subject.c_str(), _("From"), a->author.c_str(),
+        _("Date"), date, _("Message-ID"), a->message_id.c_str(),
+        _("Lines"), a->get_line_count(), _("Bytes"), a->get_byte_count(),
+        msg, s.str().c_str());
+    g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
+    gtk_widget_show_all (w);
+
+    // cleanup
+    g_free (date);
+  }
+}
+
 void GUI :: do_quit ()
 {
   gtk_main_quit ();
@@ -1357,8 +1443,11 @@ void GUI :: do_read_selected_group ()
     _data.get_group_counts (group, unread, total);
     if (!total)
       activate_action ("download-headers");
-    else if (_prefs.get_flag("get-new-headers-when-entering-group", true))
+    else if (_prefs.get_flag("get-new-headers-when-entering-group", true)) {
+      if (_prefs.get_flag ("mark-group-read-before-xover", false))
+        _data.mark_group_read (group);
       _queue.add_task (new TaskXOver (_data, group, TaskXOver::NEW), Queue::TOP);
+    }
   }
 }
 void GUI :: do_mark_selected_groups_read ()
@@ -1376,8 +1465,12 @@ void GUI :: do_clear_selected_groups ()
 void GUI :: do_xover_selected_groups ()
 {
   const quarks_t group_names (_group_pane->get_full_selection ());
-  foreach_const (quarks_t, group_names, it)
+  const bool mark_read (_prefs.get_flag ("mark-group-read-before-xover", false));
+  foreach_const (quarks_t, group_names, it) {
+    if (mark_read)
+      _data.mark_group_read (*it);
     _queue.add_task (new TaskXOver (_data, *it, TaskXOver::NEW), Queue::TOP);
+  }
 }
 
 void GUI :: do_xover_subscribed_groups ()
@@ -1385,8 +1478,12 @@ void GUI :: do_xover_subscribed_groups ()
   typedef std::vector<Quark> quarks_v;
   quarks_v groups;
   _data.get_subscribed_groups (groups);
-  foreach_const_r (quarks_v, groups, it)
+  const bool mark_read (_prefs.get_flag ("mark-group-read-before-xover", false));
+  foreach_const_r (quarks_v, groups, it) {
+    if (mark_read)
+      _data.mark_group_read (*it);
     _queue.add_task (new TaskXOver (_data, *it, TaskXOver::NEW), Queue::TOP);
+  }
 }
 
 void GUI :: do_download_headers ()
@@ -1397,7 +1494,7 @@ void GUI :: do_download_headers ()
 
 void GUI :: do_refresh_groups ()
 {
-  std::vector<Task*> tasks;
+  Queue::tasks_t tasks;
 
   const quarks_t servers (_data.get_servers ());
   foreach_const_r (quarks_t, servers, it)
