@@ -105,6 +105,7 @@ namespace pan
       virtual void do_read_previous_thread ();
       virtual void do_read_parent_article ();
       virtual void do_show_servers_dialog ();
+      virtual void do_show_selected_article_info ();
       virtual void do_plonk ();
       virtual void do_watch ();
       virtual void do_ignore ();
@@ -113,6 +114,7 @@ namespace pan
       virtual void do_cancel_article ();
       virtual void do_supersede_article ();
       virtual void do_delete_article ();
+      virtual void do_clear_article_cache ();
       virtual void do_mark_article_read ();
       virtual void do_mark_article_unread ();
       virtual void do_post ();
@@ -146,6 +148,9 @@ namespace pan
       virtual void do_unsubscribe_selected_groups ();
       virtual void do_set_charset (const char *);
 
+    public:
+      static std::string prompt_user_for_save_path (GtkWindow * parent, const Prefs& prefs);
+
 
     private: // Queue::Listener
       friend class Queue;
