@@ -248,10 +248,6 @@ HeaderPane :: render_score (GtkTreeViewColumn * col,
                           NULL);
 }
 
-#define KIBI 1024
-#define MEBI 1048576
-#define GIBI 1073741824
-
 void
 HeaderPane :: render_bytes (GtkTreeViewColumn * col,
                             GtkCellRenderer   * renderer,
@@ -262,15 +258,19 @@ HeaderPane :: render_bytes (GtkTreeViewColumn * col,
   unsigned long bytes (0);
   gtk_tree_model_get (model, iter, COL_BYTES, &bytes, -1);
 
+  static const unsigned long KIBI (1024ul);
+  static const unsigned long MEBI (1048576ul);
+  static const unsigned long GIBI (1073741824ul);
+
   char buf[128];
   if (bytes < KIBI)
-    g_snprintf (buf, sizeof(buf), _("%lu B"), bytes);
+    g_snprintf (buf, sizeof(buf), "%lu B", bytes);
   else if (bytes < MEBI)
-    g_snprintf (buf, sizeof(buf), _("%.1f KiB"), (double)bytes/KIBI);
+    g_snprintf (buf, sizeof(buf), "%.0f KiB", (double)bytes/KIBI);
   else if (bytes < GIBI)
-    g_snprintf (buf, sizeof(buf), _("%.1f MiB"), (double)bytes/MEBI);
+    g_snprintf (buf, sizeof(buf), "%.1f MiB", (double)bytes/MEBI);
   else
-    g_snprintf (buf, sizeof(buf), _("%.1f GiB"), (double)bytes/GIBI);
+    g_snprintf (buf, sizeof(buf), "%.2f GiB", (double)bytes/GIBI);
 
   g_object_set (renderer, "text", buf, NULL);
 }
@@ -514,9 +514,44 @@ namespace
   }
 }
 
+struct HeaderPane::SelectFirstArticle: public PanTreeStore::WalkFunctor
+{
+  GtkTreeView * tree_view;
+  GtkTreeSelection * tree_selection;
+  const quarks_t mids;
+  const bool do_scroll;
+  SelectFirstArticle (GtkTreeView * v, GtkTreeSelection *sel, const quarks_t& m, bool scroll):
+    tree_view(v), tree_selection(sel), mids(m), do_scroll(scroll) {}
+  virtual ~SelectFirstArticle () {}
+  virtual bool operator()(PanTreeStore *store, PanTreeStore::Row* r, GtkTreeIter *iter, GtkTreePath *unused) {
+    Row * row (dynamic_cast<Row*>(r));
+    const Article * article (row->article);
+    if (mids.count (article->message_id)) {
+      GtkTreePath * path = gtk_tree_model_get_path (GTK_TREE_MODEL(store), iter);
+      gtk_tree_view_expand_row (tree_view, path, true);
+      gtk_tree_view_expand_to_path (tree_view, path);
+      if (do_scroll) {
+        gtk_tree_view_set_cursor (tree_view, path, NULL, false);
+        gtk_tree_view_scroll_to_cell (tree_view, path, NULL, true, 0.5f, 0.0f);
+        gtk_tree_selection_select_path (tree_selection, path);
+      }
+      gtk_tree_path_free (path);
+      return false;
+    }
+    return true;
+  }
+};
+
 void
 HeaderPane :: rebuild ()
 {
+  quarks_t selectme;
+  if (1) {
+    const articles_t old_selection (get_full_selection ());
+    foreach_const (articles_t, old_selection, it)
+      selectme.insert ((*it)->message_id);
+  }
+
   _mid_to_row.clear ();
   _tree_store = build_model (_group, _atree, NULL);
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(_tree_store),
@@ -530,6 +565,12 @@ HeaderPane :: rebuild ()
 
   if (_prefs.get_flag ("expand-threads-when-entering-group", false))
     gtk_tree_view_expand_all (view);
+
+  if (!selectme.empty()) {
+    GtkTreeSelection * sel = gtk_tree_view_get_selection (view);
+    SelectFirstArticle walker (view, sel, selectme, true);
+    _tree_store->walk (walker);
+  }
 }
 
 bool
@@ -627,45 +668,17 @@ namespace
     }
   };
 
-  struct RememberArticles: public RowActionFunctor
+  struct RememberMessageId: public RowActionFunctor
   {
-    articles_t& articles;
-    RememberArticles (articles_t& a): articles(a) {}
-    virtual ~RememberArticles() {}
+    quarks_t& mids;
+    RememberMessageId (quarks_t& m): mids(m) {}
+    virtual ~RememberMessageId() {}
     virtual void operator() (GtkTreeModel* model, GtkTreeIter* it, const Article& article) {
-      articles.insert (&article);
+      mids.insert (article.message_id);
     }
   };
 }
 
-struct HeaderPane::SelectFirstArticle: public PanTreeStore::WalkFunctor
-{
-  GtkTreeView * tree_view;
-  GtkTreeSelection * tree_selection;
-  const articles_t& articles;
-  const bool do_scroll;
-  SelectFirstArticle (GtkTreeView * v, GtkTreeSelection *sel, const articles_t& a, bool scroll):
-    tree_view(v), tree_selection(sel), articles(a), do_scroll(scroll) {}
-  virtual ~SelectFirstArticle () {}
-  virtual bool operator()(PanTreeStore *store, PanTreeStore::Row* r, GtkTreeIter *iter, GtkTreePath *unused) {
-    Row * row (dynamic_cast<Row*>(r));
-    const Article * article (row->article);
-    if (articles.count (article)) {
-      GtkTreePath * path = gtk_tree_model_get_path (GTK_TREE_MODEL(store), iter);
-      gtk_tree_view_expand_row (tree_view, path, true);
-      gtk_tree_view_expand_to_path (tree_view, path);
-      if (do_scroll) {
-        gtk_tree_view_set_cursor (tree_view, path, NULL, false);
-        gtk_tree_view_scroll_to_cell (tree_view, path, NULL, true, 0.5f, 0.0f);
-        gtk_tree_selection_select_path (tree_selection, path);
-      }
-      gtk_tree_path_free (path);
-      return false;
-    }
-    return true;
-  }
-};
-
 void
 HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
 {
@@ -676,7 +689,7 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
   const bool rows_added_or_removed (!diffs.added.empty()
                                  || !diffs.reparented.empty()
                                  || !diffs.removed.empty());
-  articles_t selectme;
+  quarks_t selectme;
   if (rows_added_or_removed)
   {
     // we might need to change the selection after the update.
@@ -692,7 +705,7 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
       // so select the first article after the
       // deleted ones...
       ArticleIsNotInSet tester (diffs.removed);
-      RememberArticles actor (selectme);
+      RememberMessageId actor (selectme);
       action_next_if (tester, actor);
     }
   }
@@ -725,16 +738,18 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
 
   // removed...
   if (!diffs.removed.empty()) {
-    PanTreeStore::rows_t rows;
-    rows.reserve (diffs.removed.size());
-    foreach_const (quarks_t, diffs.removed, qit) {
-      mid_to_row_t::iterator it (_mid_to_row.find (*qit));
-      if (it!=_mid_to_row.end()) {
-        rows.push_back (*it);
-        _mid_to_row.erase (it);
-      }
-    }
-    _tree_store->remove (rows);
+    RowLessThan o;
+    std::vector<Row*> keep;
+    PanTreeStore::rows_t kill;
+    std::set_difference (_mid_to_row.begin(), _mid_to_row.end(),
+                         diffs.removed.begin(), diffs.removed.end(),
+                         inserter (keep, keep.begin()), o);
+    std::set_difference (_mid_to_row.begin(), _mid_to_row.end(),
+                         keep.begin(), keep.end(),
+                         inserter (kill, kill.begin()), o);
+    g_assert (keep.size() + kill.size() == _mid_to_row.size());
+    _tree_store->remove (kill);
+    _mid_to_row.get_container().swap (keep);
   }
 
   // changed...
@@ -1115,12 +1130,16 @@ HeaderPane :: rebuild_filter (const std::string& text, int mode)
   typedef std::pair<int,int> range_t;
   std::vector<range_t> ranges;
   ranges.reserve (6);
-  if (_action_manager.is_action_active("match-ignored-articles")) ranges.push_back (range_t(INT_MIN, -9999));
-  if (_action_manager.is_action_active("match-low-scoring-articles")) ranges.push_back (range_t(-9998, -1));
-  if (_action_manager.is_action_active("match-normal-scoring-articles")) ranges.push_back (range_t(0, 0));
-  if (_action_manager.is_action_active("match-medium-scoring-articles")) ranges.push_back (range_t(1, 4999));
-  if (_action_manager.is_action_active("match-high-scoring-articles")) ranges.push_back (range_t(5000, 9998));
-  if (_action_manager.is_action_active("match-watched-articles")) ranges.push_back (range_t(9999, INT_MAX));
+  if (_action_manager.is_action_active("match-only-watched-articles"))
+    ranges.push_back (range_t(9999, INT_MAX));
+  else {
+    if (_action_manager.is_action_active("match-ignored-articles")) ranges.push_back (range_t(INT_MIN, -9999));
+    if (_action_manager.is_action_active("match-low-scoring-articles")) ranges.push_back (range_t(-9998, -1));
+    if (_action_manager.is_action_active("match-normal-scoring-articles")) ranges.push_back (range_t(0, 0));
+    if (_action_manager.is_action_active("match-medium-scoring-articles")) ranges.push_back (range_t(1, 4999));
+    if (_action_manager.is_action_active("match-high-scoring-articles")) ranges.push_back (range_t(5000, 9998));
+    if (_action_manager.is_action_active("match-watched-articles")) ranges.push_back (range_t(9999, INT_MAX));
+  }
   for (size_t i=0; !ranges.empty() && i<ranges.size()-1; ) {
     if (ranges[i].second+1 != ranges[i+1].first)
       ++i;
@@ -1132,7 +1151,7 @@ HeaderPane :: rebuild_filter (const std::string& text, int mode)
 
 //for (size_t i=0; i<ranges.size(); ++i) std::cerr << LINE_ID << " range [" << ranges[i].first << "..." << ranges[i].second << "]" << std::endl;
 
-  std::vector<FilterInfo> filters;
+  std::deque<FilterInfo> filters;
   for (size_t i=0; i<ranges.size(); ++i) {
     const range_t& range (ranges[i]);
     const bool low_bound (range.first == INT_MIN);
@@ -1532,8 +1551,10 @@ HeaderPane :: create_filter_entry ()
   gtk_container_add (GTK_CONTAINER(search_mode_button), image);
   gtk_button_set_relief (GTK_BUTTON(search_mode_button), GTK_RELIEF_NONE);
 
-  GtkTooltips * tips = gtk_tooltips_new ();
   GtkWidget * w = gtk_button_new ();
+  GtkTooltips * tips = gtk_tooltips_new ();
+  g_object_ref_sink_pan (G_OBJECT(tips));
+  g_object_weak_ref (G_OBJECT(w), (GWeakNotify)g_object_unref, tips);
   g_object_set_data (G_OBJECT(w), "entry", entry);
   g_object_set_data (G_OBJECT(entry), "reset-button", w);
   gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
@@ -1543,11 +1564,6 @@ HeaderPane :: create_filter_entry ()
   gtk_tooltips_set_tip (GTK_TOOLTIPS(tips), w, _("Clear the Filter"), NULL);
   gtk_widget_set_sensitive (w, false);
 
-  // when the button's destroyed, free the tips too
-  g_object_ref (tips);
-  gtk_object_sink (GTK_OBJECT (tips));
-  g_signal_connect_swapped (w, "destroy", G_CALLBACK(g_object_unref), tips);
-
   GtkWidget * box = gtk_hbox_new (false, 0);
   gtk_box_pack_start (GTK_BOX(box), search_mode_button, false, false, 0);
   gtk_box_pack_start (GTK_BOX(box), entry, true, true, 0);
