@@ -43,9 +43,32 @@ using namespace pan;
 
 namespace
 {
+  GMainLoop * nongui_gmainloop (0);
+
+  void mainloop ()
+  {
+#if 1
+    if (nongui_gmainloop)
+      g_main_loop_run (nongui_gmainloop);
+    else
+      gtk_main ();
+#else
+    while (gtk_events_pending ())
+      gtk_main_iteration ();
+#endif
+  }
+
+  void mainloop_quit ()
+  {
+    if (nongui_gmainloop)
+      g_main_loop_quit (nongui_gmainloop);
+    else
+      gtk_main_quit ();
+  }
+
   gboolean delete_event_cb (GtkWidget *w, GdkEvent *e, gpointer user_data)
   {
-    gtk_main_quit ();
+    mainloop_quit ();
     return true; // don't invoke the default handler that destroys the widget
   }
 
@@ -54,7 +77,7 @@ namespace
   {
     std::cerr << "shutting down pan." << std::endl;
     signal (signum, SIG_DFL);
-    gtk_main_quit ();
+    mainloop_quit ();
   }
 #endif // G_OS_WIN32
 
@@ -128,7 +151,7 @@ namespace
       }
 
       register_shutdown_signals ();
-      gtk_main ();
+      mainloop ();
       g_signal_handler_disconnect (window, delete_cb_id);
     }
 
@@ -144,12 +167,12 @@ _("General Options\n"
 "URL Options\n"
 "  news:message-id          Show the specified article.\n"
 "  news:group.name          Show the specified newsgroup.\n"
-"  --nogui                  On news:message-id, dump the article to stdout.\n"
+"  --no-gui                 On news:message-id, dump the article to stdout.\n"
 "\n"
 "NZB Batch Options\n"
 "  --nzb file1 file2 ...    Process nzb files without launching all of Pan.\n"
 "  -o path, --output=path   Path to save attachments listed in the nzb files.\n"
-"  --nogui                  Only show console output, not the download queue.\n") << std::endl;
+"  --no-gui                 Only show console output, not the download queue.\n") << std::endl;
   }
 }
 
@@ -166,42 +189,37 @@ main (int argc, char *argv[])
 
   bool gui(true), nzb(false);
   std::string url;
+  std::string nzb_output_path;
   typedef std::vector<std::string> strings_t;
   strings_t nzb_files;
 
-  char * pch = g_get_current_dir ();
-  std::string nzb_output_path = pch;
-  g_free (pch);
-
   for (int i=1; i<argc; ++i)
   {
     const char * tok (argv[i]);
     if (!memcmp(tok,"news:", 5))
       url = tok;
-    else if (!strcmp(tok,"--nogui")) 
+    else if (!strcmp(tok,"--no-gui") || !strcmp(tok,"--nogui"))
       gui = false;
     else if (!strcmp (tok, "--debug"))
       _debug_flag = true;
     else if (!strcmp (tok, "--nzb"))
       nzb = true;
+    else if (!strcmp (tok, "--version"))
+      { std::cerr << "Pan " << VERSION << '\n'; return 0; }
     else if (!strcmp (tok, "-o") && i<argc-1) 
       nzb_output_path = argv[++i];
     else if (!memcmp (tok, "--output=", 9))
       nzb_output_path = tok+9;
     else if (!strcmp(tok,"-h") || !strcmp(tok,"--help"))
       { usage (); return 0; }
-    else
+    else {
+      nzb = true;
       nzb_files.push_back (tok);
+    }
   }
 
-#if 0
-  if (nzb && nzb_files.empty()) {
-    std::cerr << _("Error: nzb arguments used without nzb files.") << std::endl;
-    return 0;
-  }
-#endif
   if (!gui && nzb_files.empty() && url.empty()) {
-    std::cerr << _("Error: --nogui used without nzb files or news:message-id.") << std::endl;
+    std::cerr << _("Error: --no-gui used without nzb files or news:message-id.") << std::endl;
     return 0;
   }
 
@@ -232,6 +250,13 @@ main (int argc, char *argv[])
 
     if (nzb)
     {
+      // if no save path was specified, either prompt for one or
+      // use the user's home directory as a fallback.
+      if (nzb_output_path.empty() && gui)
+        nzb_output_path = GUI::prompt_user_for_save_path (NULL, prefs);
+      if (nzb_output_path.empty()) // user pressed `cancel' when prompted
+        return 0;
+
       // load the nzb files...
       std::vector<Task*> tasks;
       foreach_const (strings_t, nzb_files, it)
@@ -246,14 +271,11 @@ main (int argc, char *argv[])
         gtk_widget_show_all (w);
         g_signal_connect (w, "destroy", G_CALLBACK(destroy_cb), 0);
         g_signal_connect (G_OBJECT(w), "delete-event", G_CALLBACK(delete_event_cb), 0);
-        register_shutdown_signals ();
-        gtk_main (); // FIXME: never exits when task pane user hits 'close'
       } else {
-        GMainLoop * main_loop = g_main_loop_new (NULL, false);
-        register_shutdown_signals ();
-        g_main_loop_run (main_loop); // FIXME g_loop_quit when queue.empty()
-        g_main_loop_unref (main_loop);
+        nongui_gmainloop = g_main_loop_new (NULL, false);
       }
+      register_shutdown_signals ();
+      mainloop ();
     }
     else
     {
@@ -271,6 +293,9 @@ main (int argc, char *argv[])
       g_object_unref (pixbuf);
       run_pan_in_window (cache, data, queue, prefs, group_prefs, GTK_WINDOW(window));
     }
+
+    if (prefs.get_flag("clear-article-cache-on-shutdown", false))
+      cache.clear ();
   }
 
   g_mime_shutdown ();
