@@ -39,6 +39,7 @@ extern "C" {
 #include <pan/usenet-utils/utf8-utils.h>
 #include <pan/data/data.h>
 #include <pan/tasks/task-post.h>
+#include <pan/icons/pan-pixbufs.h>
 #include "pad.h"
 #include "hig.h"
 #include "post-ui.h"
@@ -461,9 +462,9 @@ PostUI :: set_charset (const StringView& charset)
   {
     _charset = new_charset;
 
-    char unique_name[512];
-    g_snprintf (unique_name, sizeof(unique_name), "charset-%*.*s", charset.len, charset.len, charset.str);
-    GtkAction * action = gtk_action_group_get_action (_agroup, unique_name);
+    std::string unique_name ("charset-");
+    unique_name.append (charset.str, charset.len);
+    GtkAction * action = gtk_action_group_get_action (_agroup, unique_name.c_str());
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION(action), true);
   }
 }
@@ -1845,8 +1846,6 @@ PostUI :: PostUI (GtkWindow    * parent,
   _group_entry_changed_id (0),
   _group_entry_changed_idle_tag (0)
 {
-  GtkTooltips * tips = gtk_tooltips_new ();
-
   g_assert (profiles.has_profiles());
   g_return_if_fail (message != 0);
 
@@ -1855,12 +1854,19 @@ PostUI :: PostUI (GtkWindow    * parent,
   g_signal_connect (_root, "delete-event", G_CALLBACK(delete_event_cb), this);
   gtk_window_set_role (GTK_WINDOW(_root), "pan-post-window");
   gtk_window_set_title (GTK_WINDOW(_root), _("Post Article"));
+  GdkPixbuf * pixbuf = gdk_pixbuf_new_from_inline (-1, icon_pan, FALSE, 0);
+  gtk_window_set_icon (GTK_WINDOW(_root), pixbuf); 
+  g_object_unref (G_OBJECT(pixbuf));
   g_object_set_data_full (G_OBJECT(_root), "post-ui", this, delete_post_ui);
   if (parent) {
     gtk_window_set_transient_for (GTK_WINDOW(_root), parent);
     gtk_window_set_position (GTK_WINDOW(_root), GTK_WIN_POS_CENTER_ON_PARENT);
   }
 
+  GtkTooltips * tips = gtk_tooltips_new ();
+  g_object_ref_sink_pan (G_OBJECT(tips));
+  g_object_weak_ref (G_OBJECT(_root), (GWeakNotify)g_object_unref, tips);
+
   // populate the window
   GtkWidget * vbox = gtk_vbox_new (false, PAD_SMALL);
   GtkWidget * menu_vbox = gtk_vbox_new (false, PAD_SMALL);
