@@ -97,14 +97,14 @@ namespace
 }
 
 void
-NNTP :: fire_done_func (Health health)
+NNTP :: fire_done_func (Health health, const StringView& response)
 {
    if (_listener)
    {
       Listener * l = _listener;
       debug ("I (" << (void*)this << ") am setting my _listener to 0");
       _listener = 0;
-      l->on_nntp_done (this, health);
+      l->on_nntp_done (this, health, response);
    }
 }
 
@@ -200,6 +200,19 @@ NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
          state = CMD_NEXT;
          break;
 
+      case NO_POSTING:
+      case POSTING_FAILED:
+         state = CMD_FAIL;
+         break;
+
+      case GROUP_NONEXISTENT: {
+         std::string host;
+         _socket->get_host (host);
+         Log::add_err_va (_("Can't find group \%s\" on server \"%s\""), line.to_string().c_str(), host.c_str());
+         state = CMD_FAIL;
+         break;
+      }
+
       case XOVER_FOLLOWS:
       case ARTICLE_FOLLOWS:
       case NEWGROUPS_FOLLOWS:
@@ -208,25 +221,12 @@ NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
          _nntp_response_text = true;
          break;
 
-      case GROUP_NONEXISTENT:
-      case NO_POSTING:
-      case POSTING_FAILED:
       case AUTH_REJECTED:
       case NO_GROUP_SELECTED:
       case ERROR_CMD_NOT_UNDERSTOOD:
       case ERROR_CMD_NOT_SUPPORTED:
-      case NO_PERMISSION: {
-         StringView tmp (_previous_command);
-         if (tmp.len>=2 && tmp.str[tmp.len-1]=='\n' && tmp.str[tmp.len-2]=='\r')
-           tmp.len -= 2;
-         Log::add_err_va (_("Error: Got \"%s\" from %s in response to \"%*.*s\""),
-           line.to_string().c_str(),
-           sock->get_address().c_str(),
-           tmp.len, tmp.len, tmp.str);
-         state = CMD_FAIL;
-         break;
-      }
-
+      case NO_PERMISSION:
+         Log::add_err_va (_("Error: %s"), line.to_string().c_str());
       case NO_SUCH_ARTICLE_NUMBER:
       case NO_SUCH_ARTICLE:
          state = CMD_FAIL;
@@ -251,11 +251,11 @@ NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
 
    bool more;
    switch (state) {
-      case CMD_FAIL: fire_done_func (FAIL); more = false; break;
-      case CMD_DONE: if (_commands.empty()) fire_done_func (OK); more = false; break;
+      case CMD_FAIL: fire_done_func (FAIL, line); more = false; break;
+      case CMD_DONE: if (_commands.empty()) fire_done_func (OK, line); more = false; break;
       case CMD_MORE: more = true; break; // keep listining for more on this command
       case CMD_NEXT: more = false; break; // no more responses on this command; wait for next...
-      case CMD_RETRY: fire_done_func (RETRY); more = false; break;
+      case CMD_RETRY: fire_done_func (RETRY, line); more = false; break;
       default: abort(); break;
    }
    return more;
@@ -264,14 +264,14 @@ NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
 void
 NNTP :: on_socket_abort (Socket * sock)
 {
-   fire_done_func (FAIL);
+   fire_done_func (FAIL, StringView());
 }
 
 void
 NNTP :: on_socket_error (Socket * sock)
 {
    _socket_error = true;
-   fire_done_func (RETRY);
+   fire_done_func (RETRY, StringView());
 }
 
 namespace
